/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.my.subscriptions.web.internal.dao.search;

import com.liferay.my.subscriptions.web.internal.util.MySubscriptionsUtil;
import com.liferay.portal.kernel.dao.search.ResultRow;
import com.liferay.portal.kernel.dao.search.ResultRowSplitter;
import com.liferay.portal.kernel.dao.search.ResultRowSplitterEntry;
import com.liferay.subscription.model.Subscription;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * @author Ambrín Chaudhary
 */
public class MySubscriptionsResultRowSplitter implements ResultRowSplitter {

	public MySubscriptionsResultRowSplitter(Locale locale) {
		_locale = locale;
	}

	@Override
	public List<ResultRowSplitterEntry> split(List<ResultRow> resultRows) {
		Map<String, List<ResultRow>> rowMap = new HashMap<>();

		for (ResultRow resultRow : resultRows) {
			Subscription subscription = (Subscription)resultRow.getObject();

			List<ResultRow> list = rowMap.computeIfAbsent(
				subscription.getClassName(), className -> new ArrayList<>());

			list.add(resultRow);
		}

		List<ResultRowSplitterEntry> resultRowSplitterEntries = new ArrayList<>(
			rowMap.size());

		for (Map.Entry<String, List<ResultRow>> entry : rowMap.entrySet()) {
			resultRowSplitterEntries.add(
				new ResultRowSplitterEntry(
					MySubscriptionsUtil.getAssetTypeDescription(
						_locale, entry.getKey()),
					entry.getValue()));
		}

		return resultRowSplitterEntries;
	}

	private final Locale _locale;

}